##############################
##2013 Prefresher
##Day 4 - More loops, functions, and apply
##Stephen Pettigrew
##Harvard University
##pettigrew@fas.harvard.edu
##############################


##### This R script discusses more types of loops,
##### functions, and apply

## Before we start, read the data by yourself!



























setwd("c:/users/stephen/dropbox/gov prefresher/2014 prefresher/r tutorials")
load("day4data.RData")



#########################################################################
#### Other types of loops
#########################################################################

## A second type of loop is a "while-loop." Instead of
## giving a while loop a set of indexes to iterate through,
## you give a while loop a logical statement. The loop
## will run until the logical statement evalues to false.

i <- 1
while(i < 50){
  print(i)
  i <- i + 1
}







## Repeat loops are another way to loop code, although not very commonly used.

i <- 1
i.sum <- 0

repeat{
  
  i.sum <- i.sum + i
  
  i <- i + 1
  
  if(i > 10){ 
    break()
  }
}

i.sum

## It's important to note that if you don't use the break() function to end
## the loop, R or your computer might freeze up because it'll be stuck in a
## string of code that's infinitely long and never ends.

## Watch!

i <- 1
i.sum <- 0

repeat{
  i.sum <- i.sum + i
  
  i <- i + 1
}

i
i.sum











#########################################################################
#### Functions
#########################################################################

## One of the most powerful parts of R is how easy it is write your own
## functions.

## Let's imagine that there wasn't a canned way to take a mean of a vector of
## numbers. We could easily write a function that does this

our.mean.function <- function(x){
  
  ## This part gets the sum of all the numbers in the x vector
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
  
  return(our.mean) 
}

our.mean.function(1:10)

## Notice that within the function we assigned objects to the
## names 'our.sum' and 'our.mean', but we can't see what those equal

our.sum ## not found
our.mean ## not found


## This is because the only thing that a user-defined function outputs
## is what is in the return() function at the end. 


## Alternatively, we don't necessarily need to return() function...

our.mean.function <- function(x){
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
}

our.mean.function(1:10)

## Notice that this function doesn't return anything, but if we store it as
## an object it will work.

a <- our.mean.function(1:10)
a



## Another way we can return the function out put is by not storing anything
## in the last line of code of the function.

our.mean.function <- function(x){
  our.sum <- sum(x)
  
  our.mean <- our.sum / length(x)
  
  our.mean
}

our.mean.function(1:10)




## Write a function that takes a vector of numbers and returns the 
## cube root divided by the natural log of each number in the vector

## Challenge: write the function in a way that checks that the denominator
## is valid, and outputs an error message if it is not valid
## Hint: don't forget properties of natural logs and different types
## of boolean operators




















example <- function(x){
  x^(1/3) / log(x)
}

example(0:5)



example.challenge <- function(x){
  if(1 %in% x){
  
    return("can't divide by zero")
  
  } else if (0 %in% x){
    
    return("can't divide by negative infinity")
  
  } else {
    
    x^(1/3) / log(x)
  }
}

example.challenge(1:5)

example.challenge(-5:0)

example.challenge(2:5)






## Debugging functions

## The browser() function is super super helpful if you have a really
## complicated function that has a bug in the code.

## Here's a function that doesn't work:

debugging.example <- function(x){
  
  ## Randomly sample 1000 numbers from a beta(1, 10) distribution
  holder <- rbeta(1000, 1, 10)
  
  ## Now keep the 50 largest values sampled and store them in a matrix
  
  ## Use the sort() function to order the numbers in holder
  holder <- sort(holder, decreasing = T)
  
  ## And now keep only the 50 largest values
  holder <- matrix(holder[1:50], 25, 2)
  
  ## Now multiply holder by x
  holder %*% x
}

test.matrix <- matrix(rnorm(100), 25, 4)
debugging.example(test.matrix) ## Gives us an error




## Now let's see how we could use the browser() function to debug.
## Note: you can also put browser() functions into for/while loops


debugging.example <- function(x){
  
  holder <- rbeta(1000, 1, 10)
  browser()
  
  holder <- sort(holder, decreasing = T)
  browser()
  
  holder <- matrix(holder[1:50], 25, 2)
  browser()
  
  holder %*% x
}

debugging.example(test.matrix) 
## this tells us that the error doesn't occur until after the 3rd browser() statement






debugging.example(test.matrix) 
## checking again, we can see that the dimensionality of the matrices aren't conformable




## So we can fix that by transposing the holder matrix at the end of the function

debugging.example <- function(x){
  
  holder <- rbeta(1000, 1, 10)
  
  holder <- sort(holder, decreasing = T)
  
  holder <- matrix(holder[1:50], 25, 2)
  
  t(holder) %*% x
  
}

debugging.example(test.matrix) ## Now it works





## What if we wanted the function to output both the multiplied matrices,
## AND the values in holder?
## How could we do this?



















## We could store them in a list at the end of the function:

debugging.example <- function(x){
  
  holder <- rbeta(1000, 1, 10)
  
  holder <- sort(holder, decreasing = T)
  
  holder <- matrix(holder[1:50], 25, 2)
  
  list(t(holder) %*% x,
       holder)
}

result <- debugging.example(test.matrix)

result[[1]] ## here's the multiplied matrices
result[[2]] ## here's the holder matrix



## There's one other way, and it's a really cool R-hack.
## When you write a function, you can store objects using
## <<- instead of just <-
## Using the double arrow stores the object to your workspace,
## so that it's accessible after the function is complete

debugging.example <- function(x){
  
  holder <- rbeta(1000, 1, 10)
  
  holder <- sort(holder, decreasing = T)
  
  holder <- matrix(holder[1:50], 25, 2)
  
  # put the <<- here to store holder
  holder <<- matrix(holder[1:50], 25, 2)
  
  t(holder) %*% x
  
}

debugging.example(test.matrix) ## Now it works

holder







## One more note on functions: never define a function within a function.
## It's terrible practice, and good programmers will think you're dumb













##############################################################
#### Apply
##############################################################

## The apply function (and other related functions) are one unique aspect
## of computing in R.

## You can think of apply() functions as working similar to a for-loop,
## where the synatax of the function is:

## apply(the object you want to do something to,
##       the dimension of the objection you're specifically interested in,
##       the function you want to do to that dimension)


## So here's a really basic example, where we use apply() to get the mean of
## each row of a big matrix

big.matrix <- matrix(NA, nrow = 500, ncol = 10000)
for(i in 1:500){
  big.matrix[i,] <- rnorm(1000, mean = rpois(1,lambda = 6))
}



rowmeans <- apply(big.matrix, 1, mean)
rowmeans

## Let's check if that's correct:

rowmeans[1] == mean(big.matrix[1,])
rowmeans[87] == mean(big.matrix[87,])








## You can also write your own functions and use them in an apply loop.
## There's two ways to do this.

## First, you can define the function right inside of the apply() syntax.
## You only want to do this if you're defining a really simple function.

## Let's get the mean divided by standard deviation of each column of big.matrix

apply(big.matrix, 2, function(x) mean(x) / sd(x))

## Alternatively, you could define the function first and then put it
## inside of the apply() syntax

mean.sd <- function(x){
  mean(x) / sd(x)
}

apply(big.matrix, 2, mean.sd)










## Aside from the simple apply() function, R also has several
## variants of apply()

## Probably the most important one is lapply().

## lapply() takes a list, and applies a function to every
## element in that list.

## Here's an example with some primary election data
## (subset from my congressional primary election data here:
## http://dx.doi.org/10.7910/DVN/26448)

## Each element in the list is a single primary election



View(data[[1]])
colnames(data[[1]])
class(data[[1]])





## Now write some code (using lapply or sapply) that outputs the vote percentage
## earned by the winning candidate























lapply(data, function(x) max(x$candpct))

## or

sapply(data, function(x) max(x$candpct))


# Note that with lists you can use lapply() or sapply()
# lapply() will output your result as a list
# sapply() will (usually) output your result as a vector







## So is it faster to use an apply() function or to use a for-loop?

## Let's go back to our original example of getting the row means of big.matrix

start <- proc.time()[3]
results <- apply(big.matrix, 1, mean)
applytime <- proc.time()[3] - start


start <- proc.time()[3]
results <- c()
for(i in 1:nrow(big.matrix)){
  results[i] <- mean(big.matrix[i,])
}
looptime <- proc.time()[3] - start

applytime
looptime ## The loop seems to be faster


## BUT, there's a canned function that's even faster:

start <- proc.time()[3]
results <- rowMeans(big.matrix)
proc.time()[3] - start

## Moral of the story is that often you're better off trying to
## find an function in R or in an R package that does what you want
## because it will often be faster.